<?php
defined('_JEXEC') or die('Restricted access');

/**
 * @author      Valérie Isaksen (www.alatak.net)
 * @copyright   Copyright (C) 2012-2017 Alatak.net. All rights reserved
 * @license     gpl-2.0.txt
 * @date        April 2017
 *
 */
jimport('joomla.log.log');
if (!class_exists('VmConfig')) {
	require(JPATH_ROOT . '/administrator/components/com_virtuemart/helpers/config.php');
}

class plgSystemAlatak_Vmpiwik extends JPlugin {

	/**
	 * Object Constructor.
	 *
	 * @access    public
	 * @param    object    The object to observe -- event dispatcher.
	 * @param    object    The configuration object for the plugin.
	 * @return    void
	 * @since    1.5
	 */
	function __construct(&$subject, $config) {


		VmConfig::loadConfig();

		// Load the language files
		$jlang = JFactory::getLanguage();
		$jlang->load('plg_system_alatak_vmpiwik', JPATH_SITE . '/plugins/system/alatak_vmpiwik', 'en-GB', true);
		$jlang->load('plg_system_alatak_vmpiwik', JPATH_SITE . '/plugins/system/alatak_vmpiwik', $jlang->getDefault(), true);
		$jlang->load('plg_system_alatak_vmpiwik', JPATH_SITE . '/plugins/system/alatak_vmpiwik', null, true);
		$plugin = JPluginHelper::getPlugin('system', 'alatak_vmpiwik');
		$this->params = new JRegistry($plugin->params);

		parent::__construct($subject, $config);
	}


	/*
	 * http://piwik.org/docs/tracking-api/
	 * http://piwik.org/docs/ecommerce-analytics/#toc-ecommerce-tracking
	 */

	public function plgVmOnUpdateOrderPayment(&$order, $old_order_status) {

		if (!class_exists('PiwikTracker')) {
			require("PiwikTracker.php");
		}
		$piwik_url = $this->getPiwikUrl();
		if (empty($piwik_url)) {
			return;
		}
		$orderconfirmed = in_array($order->order_status, (array)$this->params->get('orderconfirmed', ''));
		$ordercancelled = in_array($order->order_status, (array)$this->params->get('ordercancelled', ''));
		if (!$orderconfirmed && !$ordercancelled) {
			$this->writeLog(__FUNCTION__ . ' order_status=' . $order->order_status, JLog::DEBUG);
			return;
		}

		$site_id = $this->params->get('site_id', '1');

		$orderModel = VmModel::getModel('orders');
		$orderModelData = $orderModel->getOrder($order->virtuemart_order_id);

		$piwikTracker = new PiwikTracker($site_id, $piwik_url);
		$token_auth = $this->params->get('token_auth', '');

		$piwikTracker->setTokenAuth($token_auth);
		$piwikTracker->setIp( $orderModelData['details']['BT']->ip_address);

		if ($ordercancelled) {
			$this->writeLog(__FUNCTION__ . ' ordercancelled=' . $ordercancelled, JLog::DEBUG);
			$piwikTracker->doTrackEcommerceOrder(
				$order->order_number, // // (required) Unique Order ID
				0 // (required) Order Revenue grand total (includes tax, shipping, and subtracted discount)
			);
			return;
		}
		$this->writeLog(__FUNCTION__ . ' orderconfirmed=' . $orderconfirmed, JLog::DEBUG);
		self::trackEcommerceOrder($piwikTracker, $order);
	}

	function getPiwikUrl() {
		$piwik_url = $this->params->get('piwik_url', '');
		$piwik_url = str_replace(array('http://', 'https://'), '', $piwik_url);
		if (substr($piwik_url, -1) != "/") {
			$piwik_url = $piwik_url . '/';
		}
		if (empty($piwik_url)) {
			$this->writeLog('No piwik URL set', JLog::ERROR);
		}
		return $piwik_url;
	}


	function onAfterRender() {
		if (JFactory::getApplication()->isAdmin()) {
			return;
		}
		$piwik_url = $this->getPiwikUrl();

		if (empty($piwik_url)) {
			return;
		}
		if (!class_exists('PiwikTracker')) {
			require("PiwikTracker.php");
		}

		$site_id = $this->params->get('site_id', '1');

		$piwikCode = $this->getPiwikCode($site_id, $piwik_url);
		$body = JFactory::getApplication()->getBody();
		if ($body) {
			$body = str_ireplace("</head>", $piwikCode . "\n</head>", ltrim($body));
			JFactory::getApplication()->setBody($body);
		}


	}

	/**
	 * returns HTML string with PIWIK javascript code
	 * @return string
	 */
	public function getPiwikCode($site_id, $piwik_url) {


		$piwikCode = "<!-- PIWIK PLUGIN TRACKING CODE -->\n";
		$piwikCode .= '
<!-- Piwik -->
<script type="text/javascript">
  var _paq = _paq || [];
  _paq.push(["trackPageView"]);
  _paq.push(["enableLinkTracking"]);';
		$piwikCode .= $this->trackEcommerceAddToCart();
		$piwikCode .= $this->trackProduct();
		$piwikCode .= $this->trackCategory();
		$piwikCode .= ' (function() {
    var u="//' . $piwik_url . '";
    _paq.push(["setTrackerUrl", u+"piwik.php"]);
    _paq.push(["setSiteId", "' . $site_id . '"]);
    var d=document, g=d.createElement("script"), s=d.getElementsByTagName("script")[0]; g.type="text/javascript";
    g.defer=true; g.async=true; g.src=u+"piwik.js"; s.parentNode.insertBefore(g,s);
  })();
</script>
<noscript><img src="//' . $piwik_url . '?idsite=' . $site_id . '&rec=1" style="border:0" alt="" /></noscript>
<!-- END PIWIK TRACKING CODE -->
';


		return $piwikCode;
	}


	function trackEcommerceOrder($piwikTracker, $orderToTrack) {
		$this->writeLog(__FUNCTION__ . ' virtuemart_order_id=' . $orderToTrack->virtuemart_order_id . ' (' . $orderToTrack->order_number . ')', JLog::DEBUG);

		if (!class_exists('VirtueMartModelOrders')) {
			require(JPATH_VM_ADMINISTRATOR . '/models/orders.php');
		}
		// we set virtuemart_order_id=0 to remove the orders

		$orderModel = VmModel::getModel('orders');
		$order = $orderModel->getOrder($orderToTrack->virtuemart_order_id);
		$js = '';
		foreach ($order['items'] as $item) {
			$categoriesId = self::getProductCategoriesId($item->virtuemart_product_id);
			$categories = self::getProductCategoryNames($categoriesId);

			$piwikTracker->addEcommerceItem(
				self::getUniqueSku($item->virtuemart_product_id, $item->order_item_sku),
				$item->order_item_name,
				$categories,
				$item->product_final_price - $item->product_tax, // (recommended) Product price: Product sales. Excludes tax, shipping and discount
				$item->product_quantity // (optional, default to 1) Product quantity
			);
		}

		$piwikTracker->doTrackEcommerceOrder(
			$orderToTrack->order_number, // // (required) Unique Order ID
			$orderToTrack->order_total, // (required) Order Revenue grand total (includes tax, shipping, and subtracted discount)
			$orderToTrack->order_subtotal, // (optional) Order sub total (excludes shipping), Sub total amount, typically the sum of items prices for all items in this order (before Tax and Shipping costs are applied)
			$orderToTrack->order_tax, // (optional) Tax amount
			$orderToTrack->order_shipment + $orderToTrack->order_shipment_tax, // (optional) Shipping amount
			$orderToTrack->coupon_discount // (optional) Discount offered (set to false for unspecified parameter)
		);

	}

	function trackEcommerceAddToCart() {

		if (vRequest::getCmd('view') != 'cart') {
			return NULL;
		}
		if (!class_exists('VirtueMartCart')) {
			require(JPATH_VM_SITE . DS . 'helpers/cart.php');
		}
		$result = '';
		$cart = VirtueMartCart::getCart();
		if (!$cart->products) {
			// it happens on cash on delivery order done for example
			return;
		}
		foreach ($cart->products as $key => $product) {
			$productCategoryNames = self::getProductCategoryNames($product->categories);

			$sku = self::getUniqueSku($product->virtuemart_product_id, $product->product_sku);
			$product_name = addslashes(strip_tags($product->product_name));
			$categoriesjs = "[" . addslashes(implode(",", $productCategoryNames)) . "]";
			$price = self::getProductPrice($cart->cartPrices[$key]);
			$result .= '_paq.push(["addEcommerceItem","' . $product_name . '","test",' . $price . ',' . $product->quantity . ']);' . "\n";
			$this->writeLog(__FUNCTION__ . ' addEcommerceItem price:' . $price . ' quantity:' . $product->quantity, JLog::DEBUG);

		}
		// Records the cart for this visit
		$result .= "_paq.push(['trackEcommerceCartUpdate','" . $cart->cartPrices['billTotal'] . "']); \n";
		$result .= "_paq.push(['trackPageView']);\n";

		$this->writeLog(__FUNCTION__ . ' trackEcommerceCartUpdate Bill Total ' . $cart->cartPrices['billTotal'], JLog::DEBUG);


		return $result;

	}

	static function getProductPrice($pricesUnformatted) {
		return $pricesUnformatted['salesPrice'] - $pricesUnformatted['taxAmount'];

	}

	static function getProductCategoryNames($categories_id, $limit = 5) {

		$i = 0;
		$db = JFactory::getDBO();
		if (isset($categories_id)) {
			foreach ($categories_id as $category_id) {
				$query = $db->getQuery(true)
					->select($db->quoteName('category_name'))
					->from($db->quoteName('#__virtuemart_categories' . '_' . VmConfig::$vmlang) . ' AS cl ')
					->join('LEFT', $db->quoteName('#__virtuemart_categories') . ' AS c ON c.virtuemart_category_id= cl.virtuemart_category_id')
					->where($db->quoteName('c.virtuemart_category_id') . ' =  ' . $category_id);

				$db->setQuery($query);
				$categories_name[$i] = $db->loadResult();
				if ($i == $limit) {
					break;
				}
				$i++;
			}
			return $categories_name;
		} else {
			return FALSE;
		}
	}


	static function getProductCategoriesId($virtuemart_product_id, $limit = 5) {

		$db = JFactory::getDBO();

		$query = $db->getQuery(true)
			->select($db->quoteName('virtuemart_category_id'))
			->from($db->quoteName('#__virtuemart_product_categories'))
			->where($db->quoteName('virtuemart_product_id') . ' =  ' . $virtuemart_product_id);

		$db->setQuery($query);
		$categoriesId = $db->loadResultArray();
		return $categoriesId;
	}

	function trackProduct() {

		if (vRequest::getCmd('view') != 'productdetails') {
			return NULL;
		}
		$productModel = VmModel::getModel('product');

		$virtuemart_product_idArray = vRequest::getInt('virtuemart_product_id', 0);
		if (is_array($virtuemart_product_idArray) and count($virtuemart_product_idArray) > 0) {
			$virtuemart_product_id = (int)$virtuemart_product_idArray[0];
		} else {
			$virtuemart_product_id = (int)$virtuemart_product_idArray;
		}
		if (!$virtuemart_product_id) {
			$this->writeLog(__FUNCTION__ . ' setEcommerceView ' . var_export($_REQUEST, true), JLog::ERROR);
			return;
		}
		$product = $productModel->getProduct($virtuemart_product_id);

		$categoryModel = VmModel::getModel('category');
		$category = $categoryModel->getCategory($product->virtuemart_category_id);
		$sku = self::getUniqueSku($virtuemart_product_id, $product->product_sku);
		$product_name = addslashes($product->product_name);
		$result = "\n";
		//$result .= 'piwikTracker.setEcommerceView("' . self::getUniqueSku($virtuemart_product_id, $product->product_sku) . '", "' . addslashes($product->product_name) . '","' . addslashes($category->category_name) . '",' . $product->prices['salesPrice'] . ');';
		if (!isset($product->prices['salesPrice'])) {
			$price = 0;
		} else {
			$price = $product->prices['salesPrice'];
		}
		$result .= "_paq.push(['setEcommerceView', '" . $sku . "'  ,  '" . $product_name . "'  , '" . addslashes($category->category_name) . "', $price ]);\n";
		$result .= "_paq.push(['trackPageView']);\n";

		$this->writeLog(__FUNCTION__ . ' setEcommerceView ' . $sku . ' ' . $product_name . ' ' . addslashes($category->category_name) . ' ' . $price, JLog::DEBUG);

		return $result;
	}

	function trackCategory() {

		$virtuemart_category_id = vRequest::getInt('virtuemart_category_id', 0);
		if (vRequest::getCmd('view') != 'category' or $virtuemart_category_id == 0) {
			return NULL;
		}

		$categoryModel = VmModel::getModel('category');
		$category = $categoryModel->getCategory($virtuemart_category_id);
		$result = "_paq.push(['setEcommerceView', false  ,  false  , '" . addslashes($category->category_name) . "' ]);\n";
		$result .= "_paq.push(['trackPageView']);\n";

		$this->writeLog(__FUNCTION__ . ' setEcommerceView ' . $category->category_name, JLog::DEBUG);


		return $result;
	}

	function getUniqueSku($product_id, $product_sku) {
		return "(" . $product_id . ")" . $product_sku;
	}

	function writeLog($message, $error_level) {
		static $addLogger;

		jimport('joomla.log.log');

		if (!isset($addLogger)) {
			$jlogLevel = JLog::CRITICAL;
			$log = $this->params->get('log', '0');
			if ($log) {
				$jlogLevel = JLog::ALL;
			}

			JLog::addLogger(array('text_file' => 'plg_system_alatak_vmpiwik.log.php'), $jlogLevel, $this->_name);
		}


		JLog::add($message, $error_level, $this->_name);

	}
}
